package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.ArrayList;
import java.util.List;

public class ExampleMain {

    public static void main(String[] args) throws Exception {
        EntityManagerFactory emf =
                Persistence.createEntityManagerFactory("example-unit");
        try {
            persistEntities(emf);
            runProjectQuery(emf);
        } finally {
            emf.close();
        }
    }

    private static void persistEntities(EntityManagerFactory emf) throws Exception {
        System.out.println("-- Persisting entities --");
        EntityManager em = emf.createEntityManager();
        em.getTransaction().begin();
        for (Project project : createProjects()) {
            em.persist(project);
        }
        em.getTransaction().commit();
    }

    private static void runProjectQuery(EntityManagerFactory emf) {
        System.out.println("-- running query --");
        EntityManager em = emf.createEntityManager();
        List<Project> entityAList = em
                .createQuery("SELECT DISTINCT p FROM Project p JOIN TREAT(p.employees AS FullTimeEmployee) e"
                        + " WHERE e.annualSalary > 100000 ")
                .getResultList();
        entityAList.forEach(System.out::println);
        em.close();
    }


    private static List<Project> createProjects() {
        List<Project> projects = new ArrayList<>();

        FullTimeEmployee e1 = new FullTimeEmployee();
        e1.setName("Sara");
        e1.setAnnualSalary(120000);
        PartTimeEmployee e2 = new PartTimeEmployee();
        e2.setName("Jon");
        e2.setWeeklySalary(900);
        ContractEmployee e3 = new ContractEmployee();
        e3.setName("Tom");
        e3.setHourlyRate(60);
        projects.add(Project.create("Trade UI", e1, e2, e3));


        FullTimeEmployee e4 = new FullTimeEmployee();
        e4.setName("Mike");
        e4.setAnnualSalary(80000);
        PartTimeEmployee e5 = new PartTimeEmployee();
        e5.setName("Jackie");
        e5.setWeeklySalary(1200);
        ContractEmployee e6 = new ContractEmployee();
        e6.setName("Aly");
        e6.setHourlyRate(90);
        projects.add(Project.create("Broker UI", e4, e5, e6));

        return projects;
    }
}